# Drawing waveform and mass position
# TODO: add more descriptive progress message
from __future__ import annotations

from typing import List

from PySide2 import QtCore, QtWidgets

from sohstationviewer.view.plotting.waveform_processor import \
    WaveformChannelProcessor
from sohstationviewer.view.util.plot_func_names import plot_functions
from sohstationviewer.view.plotting.plotting_widget import plotting_widget

from sohstationviewer.controller.plotting_data import get_title
from sohstationviewer.controller.util import display_tracking_info

from sohstationviewer.database import extract_data

from sohstationviewer.conf import constants as const


class WaveformWidget(plotting_widget.PlottingWidget):
    """
    Widget to display waveform and mass position data.
    """

    finished = QtCore.Signal()

    def __init__(self, parent, tracking_box, *args):
        super().__init__(parent, tracking_box)

        self.data_processors: List[WaveformChannelProcessor] = []

        # Only one data processor can run at a time, so it is not a big problem
        #
        self.thread_pool = QtCore.QThreadPool()

        # Flag to indicate whether the data is being plotted for the first time
        # or the plot is being zoomed in.
        self.is_first_plotting = True

        # Used to ensure that the user cannot read a new data set when we are
        # zooming in on the waveform plot.
        self.is_working = False

    def reset_widget(self):
        """
        Reset the widget in order to plot a new data set.
        """
        self.data_processors = []
        self.is_first_plotting = True
        self.is_working = False

    def init_plot(self, start_tm, end_tm, key, data_time, time_ticks_total,
                  waveform_data, mass_pos_data):
        """
        Initialize and configure the plot.

        :param start_tm: float - requested start time to read
        :param end_tm: float - requested end time to read
        :param key: str or (str, str) station name  for mseed,
            or (device's experiment number, experiment number ) for reftek
        :param data_time: [float, float] - time range of the selected data set
        :param time_ticks_total: int - max number of tick to show on time bar
        :param waveform_data: dict - waveform data of selected data set,
            refer to DataTypeModel.__init__.waveform_data[key]['read_data']
        :param mass_pos_data: dict - mass position data of the selected
            data set, refer to DataTypeModel.__init__.mass_pos_data[key]

        """
        # The name of the plotting data is used for compatibility with the API
        # of the parent class PlottingWidget
        self.plotting_data1 = waveform_data
        self.plotting_data2 = mass_pos_data

        self.processing_log = []  # [(message, type)]
        self.gap_bar = None
        if self.axes:
            self.plotting_axes.fig.clear()
            self.draw()
        self.date_mode = self.parent.date_format.upper()
        self.time_ticks_total = time_ticks_total
        self.min_x = max(data_time[0], start_tm)
        self.max_x = min(data_time[1], end_tm)
        self.plot_total = len(self.plotting_data1) + len(self.plotting_data2)
        title = get_title(key, self.min_x, self.max_x, self.date_mode)
        self.plotting_bot = const.BOTTOM
        self.plotting_bot_pixel = const.BOTTOM_PX
        self.axes = []
        self.timestamp_bar_top = self.plotting_axes.add_timestamp_bar(0.003)
        self.plotting_axes.set_title(title)

    def plot_channels(self, start_tm, end_tm, key,
                      data_time, time_ticks_total,
                      waveform_data, mass_pos_data):
        """
        Prepare to plot waveform and mass-position data by creating a data
        processor for each channel in the waveform data (mass-position data is
        already processed when plotting SOH data). Then, run the data
        processor for each channel.

        :param start_tm: float - requested start time to read
        :param end_tm: float - requested end time to read
        :param key: str or (str, str) station name  for mseed,
            or (device's experiment number, experiment number ) for reftek
        :param data_time: [float, float] - time range of the selected data set
        :param time_ticks_total: int - max number of tick to show on time bar
        :param waveform_data: dict - waveform data of selected data set,
            refer to DataTypeModel.__init__.waveform_data[key]['read_data']
        :param mass_pos_data: dict - mass position data of the selected
            data set, refer to DataTypeModel.__init__.mass_pos_data[key]
        """
        if not self.is_working:
            self.reset_widget()
            self.is_working = True
            start_msg = 'Plotting waveform data...'
            display_tracking_info(self.tracking_box, start_msg, 'info')
            self.init_plot(start_tm, end_tm, key,
                           data_time, time_ticks_total,
                           waveform_data, mass_pos_data)
            self.create_waveform_channel_processors()
            self.process_channel()

    def get_zoom_data(self, *args, **kwargs):
        """
        Dummy method to comply with the implementation of self.set_lim in the
        base class PlottingWidget.
        """
        pass

    def set_lim(self, first_time=False):
        """
        The set_lim method of the base class PlottingWidget was not designed
        with multi-threading in mind, so it made some assumption that is
        difficult to satisfy in a multi-threaded design. While these
        assumptions do not affect the initial plotting of the data, they make
        designing a system for zooming more difficult.

        Rather than trying to comply with the design of PlottingWidget.set_lim,
        we decide to work around. This set_lim method still keeps the
        functionality of processing the data based on the zoom range. However,
        it delegates setting the new limit of the x and y axes to
        PlottingWidget.set_lim. Because PlottingWidget.set_lim processes the
        data by calling the method get_zoom_data, we implement an override that
        does nothing, thus disabling that part of PlottingWidget.set_lim.

        :param first_time: flag that indicate whether set_lim is called the
            fist time for a data set.
        """
        self.data_processors = []
        if not self.is_working:
            self.is_working = True
            start_msg = 'Zooming in...'
            display_tracking_info(self.tracking_box, start_msg, 'info')
            self.create_waveform_channel_processors()
            self.process_channel()

    def create_waveform_channel_processors(self):
        """
        Create a data processor for each channel in the waveform data.
        """
        for chan_id in self.plotting_data1:
            chan_db_info = extract_data.get_wf_plot_info(chan_id)
            if chan_db_info['plotType'] == '':
                continue
            self.plotting_data1[chan_id]['chan_db_info'] = chan_db_info
            channel_processor = WaveformChannelProcessor(
                self.plotting_data1[chan_id], chan_id, self.min_x, self.max_x,
                True
            )
            self.data_processors.append(channel_processor)
            channel_processor.finished.connect(self.process_channel)
            channel_processor.stopped.connect(self.stopped)

    def plot_mass_pos_channels(self):
        """
        Plot the mass-position data. Because mass-position data has already
        been processed in SOH widget, this method does no further processing
        and instead go straight into plotting.
        """
        for chan_id in self.plotting_data2:
            chan_db_info = extract_data.get_chan_plot_info(
                chan_id, self.parent.data_type)
            self.plotting_data2[chan_id]['chan_db_info'] = chan_db_info
            self.plot_single_channel(self.plotting_data2[chan_id], chan_id)

    @QtCore.Slot()
    def process_channel(self, channel_data=None, channel_id=None):
        """
        Process a channel of waveform data. If channel_id and channel_data is
        not None, remove the first data processor from the list of processor
        and plot the channel channel_id. No matter what, the next channel
        processor is started. If there is no more channel processor, the widget
        goes into the steps of finalizing the plot.

        Semantically, if this method is called with no argument, the processing
        is being started. Otherwise, if it receives two arguments, a data
        processor just finished its work and is signalling the widget to plot
        the processed channel and start processing the next one.

        This method has been designed to only start one processor at once. This
        is a deliberate choice in order to avoid running out of memory. Due to
        how they are designed, channels processors use a lot of memory for
        large data sets. If multiple processors are run at once, memory usage
        of the program becomes untenable. It has been observed that a 7.5 GB
        data set can cause an out of memory error.

        :param channel_data: the data of channel_id
        :param channel_id: the name of the channel to be plotted
        """
        if channel_id is not None:
            self.data_processors.pop(0)
            self.plot_single_channel(channel_data, channel_id)
        try:
            channel_processor = self.data_processors[0]
            self.thread_pool.start(channel_processor)
        except IndexError:
            self.plot_mass_pos_channels()
            if self.is_first_plotting:
                self.done()
                finished_msg = 'Waveform plot finished'
            else:
                super().set_lim()
                self.draw()
                finished_msg = 'Zooming in finished'

            display_tracking_info(self.tracking_box, finished_msg, 'info')

            self.is_working = False
            self.is_first_plotting = False

        except Exception as e:
            print(e)

    def plot_single_channel(self, c_data, chan_id):
        """
        Plot the channel chan_id.

        :param c_data: dict - data of the channel which includes down-sampled
            data in keys 'times' and 'data'. Refer to
            DataTypeModel.__init__.waveform_data[key]['read_data'][chan_id] or
            DataTypeModel.__init__.mass_pos_data[key][chan_id]
        :param chan_id: str - name of channel
        """
        if len(c_data['times']) == 0:
            return
        chan_db_info = c_data['chan_db_info']
        plot_type = chan_db_info['plotType']
        # use ax_wf because with mass position, ax has been used
        # in plottingWidget
        if 'ax_wf' not in c_data:
            ax = getattr(self.plotting, plot_functions[plot_type][1])(
                c_data, chan_db_info, chan_id, None, None)
            if ax is None:
                return
            c_data['ax_wf'] = ax
            ax.chan = chan_id
            self.axes.append(ax)
        else:
            getattr(self.plotting, plot_functions[plot_type][1])(
                c_data, chan_db_info, chan_id, c_data['ax_wf'], None)

    def done(self):
        """
        Finish up the plot after all channels in the data have been plotted.
        Also signal to the main window that the waveform plot is finished so
        that it can update its flags.
        """
        self.axes.append(self.plotting.plot_none())
        self.timestamp_bar_bottom = self.plotting_axes.add_timestamp_bar(
            0.003, top=False)
        super().set_lim(first_time=True)
        self.bottom = self.axes[-1].get_ybound()[0]
        self.ruler = self.plotting_axes.add_ruler(
            self.display_color['time_ruler'])
        self.zoom_marker1 = self.plotting_axes.add_ruler(
            self.display_color['zoom_marker'])
        self.zoom_marker2 = self.plotting_axes.add_ruler(
            self.display_color['zoom_marker'])
        # Set view size fit with the given data
        if self.main_widget.geometry().height() < self.plotting_bot_pixel:
            self.main_widget.setFixedHeight(self.plotting_bot_pixel)
        self.draw()
        self.finished.emit()

    def request_stop(self):
        """
        Request the widget to stop plotting.
        """
        # The currently running data processor will always be the first one in
        # the list.
        self.data_processors[0].request_stop()
        # Because the processors are started one at a time, we only need to
        # delete all the processors from the queue. However, we need to keep
        # the currently running data processor in memory to ensure that its
        # downsampler terminate all running threads gracefully.
        self.data_processors = [self.data_processors[0]]

    @QtCore.Slot()
    def stopped(self):
        """
        The slot that is called when the last channel processor has terminated
        all running background threads.
        """
        display_tracking_info(self.tracking_box,
                              'Waveform plot stopped', 'info')
        self.is_working = False


class WaveformDialog(QtWidgets.QWidget):
    def __init__(self, parent):
        super().__init__()
        """
        Dialog to display waveform and mass position data. Only pop up if
             'All' checkbox next to WFChans is checked or there are channels
             in WFChans textbox for mseed data or at least one of the DS
            checkbox is checked.

        :param parent: QMainWindow/QWidget - the parent widget
        """
        self.parent = parent
        """
        date_format: str - format to display date/time
        """
        self.date_format = self.parent.date_format
        """
        bit_weight_opt: str - option for bit weight ('', 'low', 'high')
            (Menu Options - Q330 Gain)
        """
        self.bit_weight_opt = self.parent.bit_weight_opt
        """
        mass_pos_volt_range_opt: str - ('regular'/'trilium'): define how to map
            values and colors when plotting
        """
        self.mass_pos_volt_range_opt = self.parent.mass_pos_volt_range_opt
        """
        data_type: str - type of data being plotted
        """
        self.data_type = None
        self.setGeometry(300, 300, 1200, 700)
        self.setWindowTitle("Raw Data Plot")

        main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(main_layout)
        main_layout.setContentsMargins(5, 5, 5, 5)
        main_layout.setSpacing(0)

        """
        info_text_browser: QTextBrowser - to display info text
        """
        self.info_text_browser = QtWidgets.QTextBrowser(self)
        """
        plotting_widget: PlottingWidget - the widget to draw waveform and
            mass position channel
        """
        self.plotting_widget = WaveformWidget(
            self, self.info_text_browser)
        self.plotting_widget.finished.connect(self.plot_finished)

        main_layout.addWidget(self.plotting_widget, 2)

        bottom_layout = QtWidgets.QHBoxLayout()
        main_layout.addLayout(bottom_layout)
        """
        save_button: save plot in plotting_widget to file
        """
        self.save_button = QtWidgets.QPushButton('Save', self)
        self.save_button.clicked.connect(self.save)
        bottom_layout.addWidget(self.save_button)
        self.info_text_browser.setFixedHeight(60)
        bottom_layout.addWidget(self.info_text_browser)

    def set_data(self, data_type, folder_name):
        """
        Set data_type and the window's title.

        :param data_type: str - data type of data being plotted
        :param folder_name: str - name of the folder of the data set to be
            displayed
        """
        self.data_type = data_type
        self.setWindowTitle("Raw Data Plot %s - %s" % (data_type, folder_name))

    def resizeEvent(self, event):
        """
        OVERRIDE Qt method.
        When WaveformDialog is resized, its plotting_widget need to initialize
            its size to fit the viewport.

        :param event: QResizeEvent - resize event
        """
        self.plotting_widget.init_size()

    @QtCore.Slot()
    def save(self):
        """
        Save the plotting to a file
        """
        print("save")

    def plot_finished(self):
        self.parent.is_plotting_waveform = False
