# UI and connectSignals for main_window

from PySide2 import QtCore, QtGui, QtWidgets

from sohstationviewer.view.calendar.calendar_widget import CalendarWidget

from sohstationviewer.view.plotting.state_of_health_widget import SOHWidget

from sohstationviewer.conf import constants


def add_separation_line(layout):
    """
    Add a line for separation to the given layout.
    :param layout: QLayout - the layout that contains the line
    """
    label = QtWidgets.QLabel()
    label.setFrameStyle(QtWidgets.QFrame.HLine | QtWidgets.QFrame.Sunken)
    label.setLineWidth(1)
    layout.addWidget(label)


class UIMainWindow(object):
    def __init__(self):
        """
        Class that create widgets, menus and connect signals for main_window.
        """
        super().__init__()
        """
        main_window: QMainWindow - Main window that contains main widget
            and menus
        """
        self.main_window = None
        """
        central_widget: QWidget - Main widget that contains all widgets
        """
        self.central_widget = None
        """
        tracking_info_text_browser: QTextBrowser - textbox that can set color
            and background color for displaying tracking info of processing
            data
        """
        self.tracking_info_text_browser = None
        # =================== top row ========================
        """
        curr_dir_button: QPushButton - Button that helps browse to the
            directory for selecting data set
        """
        self.curr_dir_button = None
        """
        curr_dir_line_edit: QLineEdit - textbox that display the current
            directory
        """
        self.curr_dir_line_edit = None
        """
        time_from_date_edit: QDateEdit - to help user select start day to read
            from the data set
        """
        self.time_from_date_edit = None
        """
        time_to_date_edit: QDateEdit - to help user select end day to read
            from the data set
        """
        self.time_to_date_edit = None
        # ======================= second row ======================
        """
        plotting_widget: PlottingWidget - Widget for plotting data
        """
        self.plotting_widget = None
        """
        open_files_list: QListWidget - Widget that display the list of
            directories in current directory for user to select
        """
        self.open_files_list = None
        """
        inside_dir_check_box: QCheckBox - to tell SOHView if current directory
            is inside the data set or the parent of the data set
        """
        self.inside_dir_check_box = None
        """
        search_line_edit: QLineEdit - textbox for user to search in
            self.open_file_list
        """
        self.search_line_edit = None
        """
        clear_button: QPushButton - clear search_line_edit
        """
        self.clear_button = None
        """
        replot_button: QPushButton - reset plotting without re-loading data
        """
        self.replot_button = None
        """
        file_list_log_check_box: QCheckBox - If checked show only ".log" files
            in open_files_list
        """
        self.file_list_log_check_box = None
        """
        file_list_zip_check_box: QCheckBox - If checked show only ".zip" files
            in open_files_list
        """
        self.file_list_zip_check_box = None
        """
        background_black_radio_button: QRadioButton - If selected,
            plotting_widget will turn to black mode
        """
        self.background_black_radio_button = None
        """
        background_white_radio_button: QRadioButton - If selected,
            plotting_widget will turn to white mode
        """
        self.background_white_radio_button = None
        """
        detect_gap_check_box: QCheckBox - If checked, gap bar will be displayed
        """
        self.detect_gap_check_box = None
        """
        gap_len_line_edit: QLineEdit - For user to enter len of gap to detect
            with unit minute (m)
        """
        self.gap_len_line_edit = None
        """
        mass_pos_123_check_box: QCheckBox: If checked, mass position 1,2,3
            will be read
        """
        self.mass_pos_123_check_box = None
        """
        mass_pos_456_check_box: QCheckBox: If checked, mass position 1,2,3
            will be read.
        """
        self.mass_pos_456_check_box = None
        """
        ds_check_boxes: [QCheckBox,] - For user to select which data stream
            to be read in RT130
        """
        self.ds_check_boxes = None
        """
        wf_all_check_box: QCheckBox - For user to select to read all waveform
            channels for mseed data
        """
        self.wf_all_check_box = None
        """
        wf_chans_line_edit: QLineEdit - For user to enter wildcard of channels
            to be read
        """
        self.wf_chans_line_edit = None
        """
        tps_check_box: QCheckBox - If checked, time-power-squared will be
            displayed in time_power_squared_dialog
        """
        self.tps_check_box = None
        """
        all_soh_chan_check_box: QCheckBox - If checked, all soh channels will
            be read and displayed
        """
        self.all_soh_chan_check_box = None
        """
        prefer_soh_chan_button: QPushButton - Button to open
            self.channel_prefer_dialog to view/add/edit/select a preferred IDs
        """
        self.prefer_soh_chan_button = None
        """
        curr_soh_ids_name_line_edit: QLineEdit - textbox to display name of
            current IDs. When all_soh_chan_check_box is checked, no name show
            up, when all_soh_chan_check_box is unchecked, current selected name
            show up.
        """
        self.curr_soh_ids_name_line_edit = None
        """
        read_button: QPushButton - Button to read data from selected directory
            from open_files_list and plot it
        """
        self.read_button = None
        """
        stop_button: QPushButton - Button to stop processing the data when it
            takes too long, for example.
        """
        self.stop_button = None
        """
        save_plot_button: QPushButton - Button to save the plots in
            self.plotting_widget to file
        """
        self.save_plot_button = None
        """
        info_list_widget: QListWidget - to list info of the read data after
            processing
        """
        self.info_list_widget = None
        # ========================= Menus =============================
        """
        form_menu: QMenu - to show all of the current open windows so that
            user can select to put a window on top of all others when need to
            look for that.
        """
        self.form_menu = None
        """
        exit_action: QAction - to exit SOHView
        """
        self.exit_action = None
        # ========================== Command Menu ======================
        """
        gps_plotter_action: QAction - to open gps_dialog to plot all the
            latitude and longitude values  of GPS position information
        """
        self.gps_plotter_action = None
        """
        log_search_action: QAction - to open log_search_dialog to search for
            lines containing the entered string
        """
        self.log_search_action = None
        # ========================== Option Menu =======================
        """
        mp_regular_color_action: QAction - set self.mass_pos_volt_range_opt to
            'regular'
        mp_trillium_color_action: QAction - set self.mass_pos_volt_range_opt to
            'trillium'
        """
        self.mp_regular_color_action = None
        self.mp_trillium_color_action = None
        """
        add_positions_to_et_action: QAction - allow to add position to ET lines
        """
        self.add_positions_to_et_action = None
        """
        Set self.date_format
        yyyy_doy_action: QAction - set format to be "YYYY:DOY"
        yyyy_mm_dd_action: QAction - set format to be "YYYY-MM-DD"
        yyyymmmdd_action: QAction - set format to be "YYYYMMDD"
        """
        self.yyyy_doy_action = None
        self.yyyy_mm_dd_action = None
        self.yyyymmmdd_action = None
        # ======================== Database Menu ==========================
        """
        add_edit_data_type_action: QAction - open DataTypes table to add/edit
        add_edit_param_action: QAction - open Parameters table to add/edit
        add_edit_channel_action: QAction - open Channels table to add/edit
        view_plot_type_action: QAction - view all plot types available and
            their description
        """
        self.add_edit_data_type_action = None
        self.add_edit_param_action = None
        self.add_edit_channel_action = None
        self.add_edit_channel_action = None
        self.view_plot_type_action = None
        # ========================= Help Menu =============================
        """
        calendar_action: QAction - Open Calendar Dialog as a helpful tool
        """
        self.calendar_action = None
        """
        about_action: QAction - Open About Dialog to give information about
            SOHView
        """
        self.about_action = None
        """
        doc_action: QAction - Open a display allowing user to browse to the
            help documents in the folder Documentation/
        """
        self.doc_action = None

    def setup_ui(self, main_window):
        """
        Setting up layout, widgets, menus for main_window
        :param main_window: QMainWindow - main GUI for user to interact with
        """
        self.main_window = main_window
        main_window.resize(1798, 1110)
        main_window.setWindowTitle("SOH Station Viewer")
        self.central_widget = QtWidgets.QWidget(main_window)
        main_window.setCentralWidget(self.central_widget)

        self.tracking_info_text_browser = QtWidgets.QTextBrowser(
            self.central_widget)

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setContentsMargins(5, 5, 5, 5)
        main_layout.setSpacing(0)
        self.central_widget.setLayout(main_layout)
        self.set_first_row(main_layout)
        self.set_second_row(main_layout)

        self.tracking_info_text_browser.setFixedHeight(60)
        main_layout.addWidget(self.tracking_info_text_browser)
        self.create_menu_bar(main_window)
        self.connect_signals(main_window)
        self.create_shortcuts(main_window)

    def set_first_row(self, main_layout):
        """
        Setting up first row with browsing directory controls and from/to time
            controls
        :param main_layout: QVBoxLayout - main layout to arrange other layouts
            from top to bottom
        """
        h_layout = QtWidgets.QHBoxLayout()
        h_layout.setContentsMargins(2, 2, 2, 2)
        h_layout.setSpacing(8)
        main_layout.addLayout(h_layout)

        self.curr_dir_button = QtWidgets.QPushButton(
            "Main Data Directory", self.central_widget)
        h_layout.addWidget(self.curr_dir_button)

        self.curr_dir_line_edit = QtWidgets.QLineEdit(
            self.central_widget)
        h_layout.addWidget(self.curr_dir_line_edit, 1)

        h_layout.addSpacing(40)

        h_layout.addWidget(QtWidgets.QLabel('From'))
        self.time_from_date_edit = QtWidgets.QDateEdit(
            self.central_widget)
        self.time_from_date_edit.setCalendarPopup(True)
        self.time_from_date_edit.setDisplayFormat("yyyy-MM-dd")
        h_layout.addWidget(self.time_from_date_edit)

        h_layout.addWidget(QtWidgets.QLabel('To'))
        self.time_to_date_edit = QtWidgets.QDateEdit(
            self.central_widget)
        self.time_to_date_edit.setCalendarPopup(True)
        self.time_to_date_edit.setDisplayFormat("yyyy-MM-dd")
        h_layout.addWidget(self.time_to_date_edit)

    def set_second_row(self, main_layout):
        """
        Setting up second rows that contains control columns and
            plotting widget for state of health
        :param main_layout: QVBoxLayout - main layout to arrange other layouts
            from top to bottom
        """
        h_layout = QtWidgets.QHBoxLayout()
        main_layout.addLayout(h_layout)
        h_layout.setContentsMargins(0, 0, 0, 0)

        left_widget = QtWidgets.QWidget(self.central_widget)
        h_layout.addWidget(left_widget)
        left_widget.setFixedWidth(240)
        left_widget.setMinimumHeight(650)
        left_layout = QtWidgets.QVBoxLayout()
        left_layout.setContentsMargins(0, 0, 0, 0)
        left_layout.setSpacing(0)
        left_widget.setLayout(left_layout)

        self.set_control_column(left_layout)

        self.plotting_widget = SOHWidget(self.main_window,
                                         self.tracking_info_text_browser,
                                         'SOHWidget')

        h_layout.addWidget(self.plotting_widget, 2)

    def set_control_column(self, left_layout):
        """
        Setting up necessary widgets for user to setup options for plotting
        :param left_layout: QVBoxLayout - layout of the left column of controls
        """
        self.open_files_list = QtWidgets.QListWidget(
            self.central_widget)
        left_layout.addWidget(self.open_files_list, 1)
        pal = self.open_files_list.palette()
        pal.setColor(QtGui.QPalette.Highlight, QtGui.QColor(128, 255, 128))
        pal.setColor(QtGui.QPalette.HighlightedText, QtGui.QColor(0, 0, 0))
        self.open_files_list.setPalette(pal)
        # allow multiple-line selection
        self.open_files_list.setSelectionMode(
            QtWidgets.QAbstractItemView.ExtendedSelection)

        self.inside_dir_check_box = QtWidgets.QCheckBox(
            'Inside Directory', self.central_widget)
        left_layout.addWidget(self.inside_dir_check_box)

        search_grid = QtWidgets.QGridLayout()
        left_layout.addLayout(search_grid)

        self.search_line_edit = QtWidgets.QLineEdit(self.central_widget)
        self.search_line_edit.setPlaceholderText('Search...')
        search_grid.addWidget(self.search_line_edit, 0, 0, 1, 3)

        self.clear_button = QtWidgets.QPushButton('Clear', self.central_widget)
        self.clear_button.setFixedWidth(65)
        search_grid.addWidget(self.clear_button, 0, 3, 1, 1)

        search_grid.addWidget(QtWidgets.QLabel('List:'), 1, 0, 1, 1)
        self.file_list_log_check_box = QtWidgets.QCheckBox(
            '.log', self.central_widget)
        search_grid.addWidget(self.file_list_log_check_box, 1, 1, 1, 1)
        self.file_list_zip_check_box = QtWidgets.QCheckBox(
            '.zip', self.central_widget)
        search_grid.addWidget(self.file_list_zip_check_box, 1, 2, 1, 1)

        self.replot_button = QtWidgets.QPushButton(
            'RePlot', self.central_widget)

        self.replot_button.setFixedWidth(95)

        search_grid.addWidget(self.replot_button, 1, 3, 1, 1)

        color_tip_fmt = ('Set the background color of the plot '
                         ' to {0}')
        background_layout = QtWidgets.QHBoxLayout()
        # background_layout.setContentsMargins(0, 0, 0, 0)
        left_layout.addLayout(background_layout)
        background_layout.addWidget(QtWidgets.QLabel('Background:     '))
        self.background_black_radio_button = QtWidgets.QRadioButton(
            'B', self.central_widget)
        self.background_black_radio_button.setToolTip(
            color_tip_fmt.format('black'))
        background_layout.addWidget(self.background_black_radio_button)
        self.background_white_radio_button = QtWidgets.QRadioButton(
            'W', self.central_widget)
        self.background_white_radio_button.setToolTip(
            color_tip_fmt.format('white'))
        background_layout.addWidget(self.background_white_radio_button)

        add_separation_line(left_layout)

        gap_layout = QtWidgets.QHBoxLayout()
        left_layout.addLayout(gap_layout)

        self.detect_gap_check_box = QtWidgets.QCheckBox(
            'DetectGap   Len:', self.central_widget)
        gap_layout.addWidget(self.detect_gap_check_box)

        self.gap_len_line_edit = QtWidgets.QLineEdit(self.central_widget)
        gap_layout.addWidget(self.gap_len_line_edit)

        gap_layout.addWidget(QtWidgets.QLabel('m'))

        mass_pos_layout = QtWidgets.QHBoxLayout()
        left_layout.addLayout(mass_pos_layout)
        mass_pos_layout.addWidget(QtWidgets.QLabel('Mass Pos:'))
        self.mass_pos_123_check_box = QtWidgets.QCheckBox(
            '123', self.central_widget)
        mass_pos_layout.addWidget(self.mass_pos_123_check_box)
        self.mass_pos_456_check_box = QtWidgets.QCheckBox(
            '456', self.central_widget)
        mass_pos_layout.addWidget(self.mass_pos_456_check_box)

        add_separation_line(left_layout)

        ds_grid = QtWidgets.QGridLayout()
        # ds_grid.setContentsMargins(0, 0, 0, 0)
        left_layout.addLayout(ds_grid)

        ds_grid.addWidget(QtWidgets.QLabel('DSs:'), 0, 0, 2, 1,
                          QtGui.Qt.AlignVCenter)
        self.ds_check_boxes = []
        count = 0
        for r in range(2):
            for c in range(4):
                count += 1
                self.ds_check_boxes.append(
                    QtWidgets.QCheckBox('%s' % count, self.central_widget))
                ds_grid.addWidget(self.ds_check_boxes[count - 1], r, c + 1)

        wf_grid = QtWidgets.QGridLayout()
        left_layout.addLayout(wf_grid)
        self.wf_all_check_box = QtWidgets.QCheckBox(
            'All  WF:', self.central_widget)
        wf_grid.addWidget(self.wf_all_check_box, 0, 0, 1, 2)

        self.wf_chans_line_edit = QtWidgets.QLineEdit(self.central_widget)
        wf_grid.addWidget(self.wf_chans_line_edit, 0, 3, 1, 2)

        self.tps_check_box = QtWidgets.QCheckBox('TPS', self.central_widget)
        wf_grid.addWidget(self.tps_check_box, 1, 0)
        add_separation_line(left_layout)

        chan_layout = QtWidgets.QHBoxLayout()
        chan_layout.setContentsMargins(0, 0, 0, 0)
        chan_layout.setSpacing(0)
        left_layout.addLayout(chan_layout)

        self.all_soh_chan_check_box = QtWidgets.QCheckBox(
            'All SOH  ', self.central_widget)
        self.all_soh_chan_check_box.setChecked(True)
        chan_layout.addWidget(self.all_soh_chan_check_box)

        self.prefer_soh_chan_button = QtWidgets.QPushButton(
            'Pref', self.central_widget)
        self.prefer_soh_chan_button.setFixedWidth(47)
        chan_layout.addWidget(self.prefer_soh_chan_button)

        chan_layout.addWidget(QtWidgets.QLabel('Cur'))
        self.curr_soh_ids_name_line_edit = QtWidgets.QLineEdit(
            self.central_widget)
        chan_layout.addWidget(self.curr_soh_ids_name_line_edit)

        submit_layout = QtWidgets.QHBoxLayout()
        submit_layout.setSpacing(5)
        left_layout.addLayout(submit_layout)
        self.read_button = QtWidgets.QPushButton('Read', self.central_widget)
        self.read_button.setToolTip('Read selected files')
        submit_layout.addWidget(self.read_button)
        self.stop_button = QtWidgets.QPushButton('Stop', self.central_widget)
        self.stop_button.setToolTip('Halt ongoing read')
        submit_layout.addWidget(self.stop_button)
        self.save_plot_button = QtWidgets.QPushButton(
            'Save plot', self.central_widget)
        self.save_plot_button.setToolTip('Save plots to disk')
        submit_layout.addWidget(self.save_plot_button)

        self.info_list_widget = QtWidgets.QListWidget(self.central_widget)
        left_layout.addWidget(self.info_list_widget, 1)

    def create_shortcuts(self, main_window):
        seq = QtGui.QKeySequence('Ctrl+F')
        chdir_shortcut = QtWidgets.QShortcut(seq, main_window)
        chdir_shortcut.activated.connect(
            main_window.change_current_directory)

    def create_menu_bar(self, main_window):
        """
        Setting up menu bar
        :param main_window: QMainWindow - main GUI for user to interact with
        """
        main_menu = main_window.menuBar()
        file_menu = main_menu.addMenu("File")
        command_menu = main_menu.addMenu("Commands")
        option_menu = main_menu.addMenu("Options")
        self.form_menu = main_menu.addMenu("Forms")
        database_menu = main_menu.addMenu("Database")
        help_menu = main_menu.addMenu("Help")

        # exitAction = QtWidgets.QAction(QtGui.QIcon('exit.png'), "Exit", self)
        # exitAction.setShortcut("Ctrl+X")
        self.create_file_menu(main_window, file_menu)
        self.create_command_menu(main_window, command_menu)
        self.create_option_menu(main_window, option_menu)

        self.create_database_menu(main_window, database_menu)
        self.create_help_menu(main_window, help_menu)

    def create_file_menu(self, main_window, menu):
        """
        Create File Menu's Actions which involve in config or close SOHView
        :param main_window: QMainWindow - main GUI for user to interact with
        :param menu: QMenu - File Menu
        """
        self.exit_action = QtWidgets.QAction('Close', main_window)
        menu.addAction(self.exit_action)

    def create_command_menu(self, main_window, menu):
        """
        Create Commands Menu's Actions to open some additional windows for user
            to interact with
        :param main_window: QMainWindow - main GUI for user to interact with
        :param menu: QMenu - Commands Menu
        """
        self.gps_plotter_action = QtWidgets.QAction(
            'GPS Plotter', main_window)
        menu.addAction(self.gps_plotter_action)

        self.log_search_action = QtWidgets.QAction(
            'Log Search', main_window)
        menu.addAction(self.log_search_action)

    def create_option_menu(self, main_window, menu):
        """
        Create Options Menu's Actions which help setting options for plotting.
        :param main_window: QMainWindow - main GUI for user to interact with
        :param menu: QMenu - Options Menu
        """
        mp_coloring_menu = QtWidgets.QMenu('MP Coloring:', main_window)
        menu.addMenu(mp_coloring_menu)
        mp_coloring_group = QtWidgets.QActionGroup(main_window)
        self.mp_regular_color_action = QtWidgets.QAction(
            '0.5, 2.0, 4.0, 7.0 (Regular)', main_window)
        self.mp_regular_color_action.setCheckable(True)
        mp_coloring_menu.addAction(self.mp_regular_color_action)
        mp_coloring_group.addAction(self.mp_regular_color_action)
        self.mp_trillium_color_action = QtWidgets.QAction(
            '0.5, 1.8, 2.4, 3.5 (Trillium)', main_window)
        self.mp_trillium_color_action.setCheckable(True)
        mp_coloring_menu.addAction(self.mp_trillium_color_action)
        mp_coloring_group.addAction(self.mp_trillium_color_action)
        menu.addSeparator()

        self.add_positions_to_et_action = QtWidgets.QAction(
            'Add Positions to ET Lines', main_window)
        self.add_positions_to_et_action.setCheckable(True)
        menu.addAction(self.add_positions_to_et_action)

        date_format_menu = QtWidgets.QMenu('Date Format:', main_window)
        menu.addMenu(date_format_menu)
        self.yyyy_doy_action = QtWidgets.QAction(
            'YYYY:DOY', main_window)
        self.yyyy_doy_action.setCheckable(True)
        date_format_menu.addAction(self.yyyy_doy_action)
        self.yyyy_mm_dd_action = QtWidgets.QAction(
            'YYYY-MM-DD', main_window)
        self.yyyy_mm_dd_action.setCheckable(True)
        date_format_menu.addAction(self.yyyy_mm_dd_action)
        self.yyyymmmdd_action = QtWidgets.QAction(
            'YYYYMMMDD', main_window)
        self.yyyymmmdd_action.setCheckable(True)
        date_format_menu.addAction(self.yyyymmmdd_action)

    def create_database_menu(self, main_window, menu):
        """
        Create Database Menu's Actions which allow user to add/edit
            some tables in database.
        :param main_window: QMainWindow - main GUI for user to interact with
        :param menu: QMenu - Database Menu
        """
        self.add_edit_data_type_action = QtWidgets.QAction(
            'Add/Edit Data Types', main_window)
        menu.addAction(self.add_edit_data_type_action)

        self.add_edit_param_action = QtWidgets.QAction(
            'Add/Edit Parameters', main_window)
        menu.addAction(self.add_edit_param_action)

        self.add_edit_channel_action = QtWidgets.QAction(
            'Add/Edit Channels', main_window)
        menu.addAction(self.add_edit_channel_action)

        self.view_plot_type_action = QtWidgets.QAction(
            'View Plot Types', main_window)
        menu.addAction(self.view_plot_type_action)

    def create_help_menu(self, main_window, menu):
        """
        Create Help Menu's Actions which involve in viewing documentations or
            helpful tools.
        :param main_window: QMainWindow - main GUI for user to interact with
        :param menu: QMenu - Help Menu
        """
        self.calendar_action = QtWidgets.QAction(
            'Calendar', main_window)
        menu.addAction(self.calendar_action)

        self.about_action = QtWidgets.QAction(
            'About', main_window)
        menu.addAction(self.about_action)

        self.doc_action = QtWidgets.QAction('Documentation', main_window)
        menu.addAction(self.doc_action)

    def connect_signals(self, main_window):
        """
        Connect widgets what they do
        :param main_window: QMainWindow - main GUI for user to interact with
        """
        self.connect_menu_signals(main_window)
        self.connect_widget_signals(main_window)

    def connect_menu_signals(self, main_window):
        """
        Connect menus' actions to what they do
        :param main_window: QMainWindow - main GUI for user to interact with
        """
        # File
        self.exit_action.triggered.connect(main_window.close)

        # Commands

        # Options
        self.mp_regular_color_action.triggered.connect(
            lambda: setattr(
                main_window, 'mass_pos_volt_range_opt', 'regular'))
        self.mp_trillium_color_action.triggered.connect(
            lambda: setattr(
                main_window, 'mass_pos_volt_range_opt', 'trillium'))

        self.yyyy_mm_dd_action.triggered.connect(
            lambda: main_window.set_date_format('yyyy-MM-dd'))
        self.yyyymmmdd_action.triggered.connect(
            lambda: main_window.set_date_format('yyyyMMMdd'))
        self.yyyy_doy_action.triggered.connect(
            lambda: main_window.set_date_format('yyyyDOY'))

        # Database
        self.add_edit_data_type_action.triggered.connect(
            main_window.open_data_type)
        self.add_edit_param_action.triggered.connect(
            main_window.open_param)
        self.add_edit_channel_action.triggered.connect(
            main_window.open_channel)
        self.view_plot_type_action.triggered.connect(
            main_window.open_plot_type)

        # Form

        # Help
        self.calendar_action.triggered.connect(main_window.open_calendar)
        self.doc_action.triggered.connect(self.open_help_browser)

    def connect_widget_signals(self, main_window):
        main_window.current_directory_changed.connect(
            self.curr_dir_line_edit.setText)

        # first Row
        self.time_from_date_edit.setCalendarWidget(CalendarWidget(main_window))
        self.time_from_date_edit.setDate(QtCore.QDate.fromString(
            constants.DEFAULT_START_TIME, QtCore.Qt.ISODate
        ))

        self.curr_dir_button.clicked.connect(
            main_window.change_current_directory)
        self.time_to_date_edit.setCalendarWidget(CalendarWidget(main_window))
        self.time_to_date_edit.setDate(QtCore.QDate.currentDate())

        # second Row
        self.open_files_list.itemDoubleClicked.connect(
            main_window.open_files_list_item_double_clicked)

        self.replot_button.clicked.connect(main_window.replot_loaded_data)

        self.all_soh_chan_check_box.clicked.connect(
            main_window.all_chan_clicked)
        self.prefer_soh_chan_button.clicked.connect(
            main_window.open_channel_preferences)
        self.read_button.clicked.connect(main_window.read_selected_files)

        self.stop_button.clicked.connect(main_window.stop)
